@@ -1,7 +1,7 @@
 
 from AccessControl import ClassSecurityInfo
 from Acquisition.interfaces import IAcquirer
-from Acquisition import aq_parent
+from Acquisition import aq_parent, aq_inner
 from App.class_init import InitializeClass
 from App.special_dtml import DTMLFile
 from Persistence import Persistent
@@ -115,16 +115,16 @@ class Field:
         if tales_expr:
             # kw are the expression namespace. Fill in it correctly.
 
-            form = aq_parent(self)
-            context = aq_parent(form)
+            field = aq_inner(self)
+            context = aq_parent(self)
             if 'here' not in kw:
-                kw['here'] = form
+                kw['here'] = field
             if 'request' not in kw and hasattr(self, 'REQUEST'):
                 kw['request'] = self.REQUEST
             kw['modules'] = SecureModuleImporter
             value = tales_expr.__of__(self)(
-                field=self,
-                form=form,
+                field=field,
+                form=aq_parent(field),
                 context=context,
                 **kw)
         else:
@@ -557,7 +557,7 @@ class ZMIField(
         """Change TALES expressions through XMLRPC.
         """
         # BEWARE: there is no validation on the values passed through the map
-        from TALESField import TALESMethod
+        from Products.Formulator.TALESField import TALESMethod
         result = {}
         for key, value in map.items():
             result[key] = TALESMethod(value)
@@ -622,20 +622,16 @@ class ZMIField(
 InitializeClass(ZMIField)
 PythonField = ZMIField # NOTE: for backwards compatibility
 
-class ZClassField(Field):
-    """Base class for a field implemented as a ZClass.
-    """
-    pass
 
-
-def field_added(the_object, event):
+def field_added(field, event):
     # update group info in form
-    if the_object != event.object:
+    if field != event.object:
         return
-    event.newParent.field_added(the_object.id)
+    event.newParent.field_added(field.id)
 
-def field_removed(the_object, event):
+def field_removed(field, event):
      # update group info in form
-    if the_object != event.object:
+    if field != event.object:
         return
-    event.oldParent.field_removed(the_object.id)
+    event.oldParent.field_removed(field.id)
+
